/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <getopt.h>
#include <readline/readline.h>
#include <readline/history.h>

#include "my_stdio.h"
#include "switch_rf.h"


/* The names of functions that actually do the manipulation. */
int com_help  __P((char *));
int com_quit  __P((char *));
int com_sw    __P((char *));
int com_by    __P((char *));
int com_nby   __P((char *));


/* A structure which contains information on the commands this program
   can understand. */

typedef struct {
	char *name;		/* User printable name of the function. */
	rl_icpfunc_t *func;	/* Function to call to do the job. */
	char *doc;		/* Documentation for this function.  */
} COMMAND;

COMMAND commands[] = {
	{ "sw", com_sw, "Set switch mode (A-H):\n\
\t\tA: Antenna 1 (0.1 - 1.1 GHz).\n\
\t\tB: Antenna 1 & calibration diode.\n\
\t\tC: Antenna 2 (1.1 - 3.0 GHz).\n\
\t\tD: Antenna 2 & calibration diode.\n\
\t\tE: Antenna 3 (3.0 - 18.0 GHz).\n\
\t\tF: Antenna 3 & calibration diode.\n\
\t\tG: Antenna 4 (18.0 - 25.0 GHz).\n\
\t\tH: Antenna 4 & calibration diode" },
//	{ "help", com_help, "Display this text" },
	{ "?", com_help, "Synonym for `help'" },
//	{ "quit", com_quit, "Quit ..." },
	{ "q", com_quit, "Quit :-)" },
	{ "by",  com_by,  "Out byte (in hexadecimal)" },
	{ "nby", com_nby, "Out negated byte (in hexadecimal)" },
	{ (char *)NULL, (rl_icpfunc_t *)NULL, (char *)NULL }
};

/* When non-zero, this means the user is done using this program. */
int done = 0;


/* Generator function for command completion.  STATE lets us
   know whether to start from scratch; without any state
   (i.e. STATE == 0), then we start at the top of the list. */
char *command_generator (char *text, int state)
{ 
	static int list_index, len;
	char *name;

	/* If this is a new word to complete, initialize now.  This
	   includes saving the length of TEXT for efficiency, and
	   initializing the index variable to 0. */
	if (!state) {
		list_index = 0;
		len = strlen (text);
	}

	/* Return the next name which partially matches from the
	   command list. */
	while ((name = commands[list_index].name)) {
		list_index++;
		if (strncmp (name, text, len) == 0)
			return (strdup(name));
	}

	/* If no names matched, then return NULL. */
	return ((char *)NULL);
}


char **fileman_completion (const char *text, int start, int end)
{
	char **matches;

	matches = (char **)NULL;

	/* If this word is at the start of the line, then it is a
	   command to complete.  Otherwise it is the name of a file in
	   the current directory.  */
	if (start == 0)
	  matches = rl_completion_matches (text, 
					   (rl_compentry_func_t *) command_generator);
	
	return (matches);
}


/* Tell the GNU Readline library how to complete.  We want to try to
   complete on command names if this is the first word in the line, or
   on filenames if not. */
void initialize_readline (void)
{
	/* Allow conditional parsing of the ~/.inputrc file. */
	rl_readline_name = "swbox";
	
	/* Tell the completer that we want a crack first. */
	rl_attempted_completion_function = fileman_completion;
}


/* Look up NAME as the name of a command, and return a pointer to that
   command.  Return a NULL pointer if NAME isn't a command name. */
COMMAND *find_command (char *name)
{
	int i;

	for (i = 0; commands[i].name; i++)
		if (strcmp (name, commands[i].name) == 0)
			return (&commands[i]);
	
	return ((COMMAND *)NULL);
}


/* Execute a command line. */
int execute_line (char *line)
{
  int i;
	COMMAND *command;
	char *word;

	/* Isolate the command word. */
	i = 0;
	while (line[i] && WHITESPACE(line[i]))	i++;
	word = line + i;

	while (line[i] && !WHITESPACE (line[i])) i++;

	if (line[i])
		line[i++] = '\0';

	command = find_command (word);

	if (!command) {
		fprintf (stderr, "%s: No such command for "".\n", word);
		return (-1);
	}

	/* Get argument to command, if any. */
	while (WHITESPACE (line[i])) i++;

	word = line + i;

	/* Call the function. */
	return ((*(command->func)) (word));
}


/* The user wishes to quit using this program.  Just set DONE
   non-zero. */
int com_quit (char *arg)
{
	done = 1;
	return (0);
}

/* Print out help for ARG, or for all of the commands if ARG is not
   present. */
int com_help (char *arg)
{
	int i;
	int printed = 0;

	for (i = 0; commands[i].name; i++) {
		if (!*arg || (strcmp (arg, commands[i].name) == 0)) {
			printf ("%s\t%s.\n", commands[i].name, 
				commands[i].doc);
			printed++;
		}
	}

	if (!printed) {
		printf ("No commands match `%s'.  Possibilties are:\n", arg);

		for (i = 0; commands[i].name; i++) {
			/* Print in six columns. */
			if (printed == 6) {
				printed = 0;
				printf ("\n");
			}
			
			printf ("%s\t", commands[i].name);
			printed++;
		}

		if (printed)
			printf ("\n");
	}
	return (0);
}



int command_main(void)
{
	char *line, *s;

	initialize_readline ();	/* Bind our completer. */

	/* Loop reading and executing lines until the user quits. */
	for ( ; done == 0; ) {
		line = readline ("swtest> ");
		
		if (!line)
			break;
		
		/* Remove leading and trailing whitespace from the
		   line.  Then, if there is anything left, add it to
		   the history list and execute it. */
		s = stripwhite (line);

		if (*s)
		{
			add_history (s);
			execute_line (s);
		}
		
		free (line);
	}
	return (0);
}


int com_sw (char *arg)
{
	char mode;

	if ( (arg[0] >= 'a' ) && (arg[0] <= 'z') ) {
		mode = arg[0] - 32;
	}else{
		mode = arg[0];
	}
	printf(" Seting mode: %c\n", mode);
	if ( antenna_switchs_set(mode) != 0) {
		printf(" Invalid mode: %s\n", arg);
	}
	return 0;
}


int com_by (char *arg)
{
	char b = (unsigned char) strtol(arg, NULL, 16);
	
	return antenna_byte_set(b);
}


int com_nby (char *arg)
{
	char b = (unsigned char) strtol(arg, NULL, 16);
	
	return antenna_byte_set(~b);
}




int main(int argc, char * const argv[])
{
	int op;
	int switch_rf_port = SWITCH_RF_PORT;

#ifdef VERBOSE_MODE
	verbose_flag = PERR;
#endif
	while  ((op = getopt(argc, argv, "vp:h")) != EOF)
		switch( op ){
#ifdef VERBOSE_MODE
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
#endif
		case 'p':
			switch_rf_port = strtol(optarg, (char **)NULL, 16);
			break;
		case 'h':
		default:
			printf("Use %s [-v (incremental)] [-p port]\n", 
			       argv[0]);
			exit(-1);
		}
	if (antenna_switchs_open(switch_rf_port) != 0) {
		SYSLOG(LOG_CRIT, "Cannot acces rf switch port.");
		exit(1);
	}
        command_main();
	antenna_switchs_close();
        return 0;
}

